@@ -244,7 +244,6 @@ static void headset_discovery_complete(struct audio_device *dev, void *user_data
 failed:
 	error("discovery failed");
 	unix_ipc_error(client, BT_SETCONFIGURATION_RSP, EIO);
-	client->dev = NULL;
 }
 
 static void headset_setup_complete(struct audio_device *dev, void *user_data)
@@ -252,34 +251,12 @@ static void headset_setup_complete(struct audio_device *dev, void *user_data)
 	struct unix_client *client = user_data;
 	char buf[BT_AUDIO_IPC_PACKET_SIZE];
 	struct bt_setconfiguration_rsp *rsp = (void *) buf;
-	struct headset_data *hs = &client->d.hs;
 
 	client->req_id = 0;
 
 	if (!dev)
 		goto failed;
 
-	switch (client->access_mode) {
-	case BT_CAPABILITIES_ACCESS_MODE_READ:
-		hs->lock = HEADSET_LOCK_READ;
-		break;
-	case BT_CAPABILITIES_ACCESS_MODE_WRITE:
-		hs->lock = HEADSET_LOCK_WRITE;
-		break;
-	case BT_CAPABILITIES_ACCESS_MODE_READWRITE:
-		hs->lock = HEADSET_LOCK_READ | HEADSET_LOCK_WRITE;
-		break;
-	default:
-		hs->lock = 0;
-		break;
-	}
-
-	if (!headset_lock(dev, hs->lock)) {
-		error("Unable to lock headset");
-		debug("Ignoring this");
-		/* goto failed; */
-	}
-
 	memset(buf, 0, sizeof(buf));
 
 	rsp->rsp_h.msg_h.msg_type = BT_SETCONFIGURATION_RSP;
@@ -296,7 +273,6 @@ static void headset_setup_complete(struct audio_device *dev, void *user_data)
 failed:
 	error("config failed");
 	unix_ipc_error(client, BT_SETCONFIGURATION_RSP, EIO);
-	client->dev = NULL;
 }
 
 static void headset_resume_complete(struct audio_device *dev, void *user_data)
@@ -305,12 +281,18 @@ static void headset_resume_complete(struct audio_device *dev, void *user_data)
 	char buf[BT_AUDIO_IPC_PACKET_SIZE];
 	struct bt_streamstart_rsp *rsp = (void *) buf;
 	struct bt_streamfd_ind *ind = (void *) buf;
+	struct headset_data *hs = &client->d.hs;
 
 	client->req_id = 0;
 
 	if (!dev)
 		goto failed;
 
+	if (!headset_lock(dev, hs->lock)) {
+		error("Unable to lock headset");
+		goto failed;
+	}
+
 	memset(buf, 0, sizeof(buf));
 
 	rsp->rsp_h.msg_h.msg_type = BT_STREAMSTART_RSP;
@@ -325,14 +307,36 @@ static void headset_resume_complete(struct audio_device *dev, void *user_data)
 
 	if (unix_sendmsg_fd(client->sock, client->data_fd) < 0) {
 		error("unix_sendmsg_fd: %s(%d)", strerror(errno), errno);
+		headset_unlock(client->dev, hs->lock);
 		goto failed;
 	}
 
 	return;
 
 failed:
-	error("resume failed");
+	error("headset_resume_complete: resume failed");
 	unix_ipc_error(client, BT_STREAMSTART_RSP, EIO);
+}
+
+static void headset_suspend_complete(struct audio_device *dev, void *user_data)
+{
+	struct unix_client *client = user_data;
+	char buf[BT_AUDIO_IPC_PACKET_SIZE];
+	struct bt_streamstart_rsp *rsp = (void *) buf;
+
+	if (!dev)
+		goto failed;
+
+	memset(buf, 0, sizeof(buf));
+	rsp->rsp_h.msg_h.msg_type = BT_STREAMSTOP_RSP;
+	rsp->rsp_h.posix_errno = 0;
+	unix_ipc_sendmsg(client, &rsp->rsp_h.msg_h);
+
+	return;
+
+failed:
+	error("suspend failed");
+	unix_ipc_error(client, BT_STREAMSTOP_RSP, EIO);
 	client->dev = NULL;
 }
 
@@ -606,6 +610,7 @@ failed:
 static void start_config(struct audio_device *dev, struct unix_client *client)
 {
 	struct a2dp_data *a2dp;
+	struct headset_data *hs;
 	unsigned int id;
 
 	client->type = select_service(dev, client->interface);
@@ -628,7 +633,25 @@ static void start_config(struct audio_device *dev, struct unix_client *client)
 		break;
 
 	case TYPE_HEADSET:
-		id = headset_request_stream(dev, headset_setup_complete, client);
+		hs = &client->d.hs;
+
+		switch (client->access_mode) {
+		case BT_CAPABILITIES_ACCESS_MODE_READ:
+			hs->lock = HEADSET_LOCK_READ;
+			break;
+		case BT_CAPABILITIES_ACCESS_MODE_WRITE:
+			hs->lock = HEADSET_LOCK_WRITE;
+			break;
+		case BT_CAPABILITIES_ACCESS_MODE_READWRITE:
+			hs->lock = HEADSET_LOCK_READ | HEADSET_LOCK_WRITE;
+			break;
+		default:
+			hs->lock = 0;
+			break;
+		}
+
+		id = headset_config_stream(dev, headset_setup_complete,
+					hs->lock, client);
 		client->cancel = headset_cancel_stream;
 		break;
 
@@ -654,6 +677,7 @@ failed:
 static void start_resume(struct audio_device *dev, struct unix_client *client)
 {
 	struct a2dp_data *a2dp;
+	struct headset_data *hs;
 	unsigned int id;
 
 	client->type = select_service(dev, client->interface);
@@ -679,15 +703,14 @@ static void start_resume(struct audio_device *dev, struct unix_client *client)
 					a2dp_resume_complete, client);
 		client->cancel = a2dp_source_cancel;
 
-		if (id == 0) {
-			error("resume failed");
-			goto failed;
-		}
-
 		break;
 
 	case TYPE_HEADSET:
-		headset_resume_complete(dev, client);
+		hs = &client->d.hs;
+
+		id = headset_request_stream(dev, headset_resume_complete,
+					hs->lock, client);
+		client->cancel = headset_cancel_stream;
 		break;
 
 	default:
@@ -695,6 +718,11 @@ static void start_resume(struct audio_device *dev, struct unix_client *client)
 		goto failed;
 	}
 
+	if (id == 0) {
+		error("start_resume: resume failed");
+		goto failed;
+	}
+
 	return;
 
 failed:
@@ -704,6 +732,7 @@ failed:
 static void start_suspend(struct audio_device *dev, struct unix_client *client)
 {
 	struct a2dp_data *a2dp;
+	struct headset_data *hs;
 	unsigned int id;
 
 	client->type = select_service(dev, client->interface);
@@ -731,7 +760,10 @@ static void start_suspend(struct audio_device *dev, struct unix_client *client)
 		break;
 
 	case TYPE_HEADSET:
-		id = headset_request_stream(dev, headset_setup_complete, client);
+		hs = &client->d.hs;
+
+		id = headset_suspend_stream(dev, headset_suspend_complete,
+					hs->lock, client);
 		client->cancel = headset_cancel_stream;
 		break;
 
@@ -920,7 +952,8 @@ static void handle_setconfiguration_req(struct unix_client *client,
 
 	client->access_mode = req->access_mode;
 
-	start_config(dev, client); 
+	start_config(dev, client);
+
 	return;
 
 failed:
