@@ -25,6 +25,8 @@ int read_device_alias(const char *src, const char *dst, char *alias, size_t size
 int write_device_alias(const char *src, const char *dst, const char *alias);
 int write_discoverable_timeout(bdaddr_t *bdaddr, int timeout);
 int read_discoverable_timeout(const char *src, int *timeout);
+int write_pairable_timeout(bdaddr_t *bdaddr, int timeout);
+int read_pairable_timeout(const char *src, int *timeout);
 int write_device_mode(bdaddr_t *bdaddr, const char *mode);
 int read_device_mode(const char *src, char *mode, int length);
 int read_on_mode(const char *src, char *mode, int length);
@@ -43,7 +45,7 @@ int write_l2cap_info(bdaddr_t *local, bdaddr_t *peer,
 int read_l2cap_info(bdaddr_t *local, bdaddr_t *peer,
 			uint16_t *mtu_result, uint16_t *mtu,
 			uint16_t *mask_result, uint32_t *mask);
-int write_version_info(bdaddr_t *local, bdaddr_t *peer, uint16_t err, uint8_t lmp_ver, uint16_t lmp_subver);
+int write_version_info(bdaddr_t *local, bdaddr_t *peer, uint16_t manufacturer, uint8_t lmp_ver, uint16_t lmp_subver);
 int write_features_info(bdaddr_t *local, bdaddr_t *peer, unsigned char *features);
 int write_lastseen_info(bdaddr_t *local, bdaddr_t *peer, struct tm *tm);
 int write_lastused_info(bdaddr_t *local, bdaddr_t *peer, struct tm *tm);
@@ -68,6 +70,8 @@ int store_device_id(const gchar *src, const gchar *dst,
 int read_device_id(const gchar *src, const gchar *dst,
 					uint16_t *source, uint16_t *vendor,
 					uint16_t *product, uint16_t *version);
+int write_device_pairable(bdaddr_t *local, gboolean mode);
+int read_device_pairable(bdaddr_t *local, gboolean *mode);
 
 #define PNP_UUID		"00001200-0000-1000-8000-00805f9b34fb"
 
